package main

import (
	"encoding/base64"
	"flag"
	"fmt"
	"log"
	"net/http"
	"os"
	"path/filepath"
	"strings"

	"code.ivysaur.me/webdir/thumbnail"
)

const (
	DEFAULT_THUMB_CACHE_LIMIT = 100
)

type WebdirApp struct {
	Basedir           string
	ListenAddr        string
	Thumbnailer       *thumbnail.Thumbnailer
	ExcludeComponents map[string]struct{}
	StaticResources   map[string]struct{}
}

func (this *WebdirApp) GetAbsPathFromRel(path string) (string, error) {
	absPath, err := filepath.Abs(filepath.Join(this.Basedir, path))
	if err != nil {
		return "", err
	}

	if !strings.HasPrefix(absPath, this.Basedir) {
		return "", fmt.Errorf("Out of base read for '%s'\n", absPath)
	}

	return absPath, nil
}

func (this *WebdirApp) WebRequest(w http.ResponseWriter, r *http.Request) {

	if r.URL.Path == "/" {
		w.Header().Set("Location", "/path/")
		w.WriteHeader(301)

	} else if len(r.URL.Path) > 8 && r.URL.Path[0:8] == "/static/" {

		this.RenderStatic(w, r.URL.Path[8:])

	} else if strings.HasPrefix(r.URL.Path, "/path/") {
		pathPart, err := base64.StdEncoding.DecodeString(r.URL.Path[6:])
		if err != nil {
			this.RenderError(w, 400, err)
			return
		}

		this.RenderPath(w, string(pathPart))

	} else if strings.HasPrefix(r.URL.Path, "/thumb/") {
		pathPart, err := base64.StdEncoding.DecodeString(r.URL.Path[7:])
		if err != nil {
			this.RenderError(w, 400, err)
			return
		}

		this.RenderThumb(w, string(pathPart))

	} else if strings.HasPrefix(r.URL.Path, "/get/") {
		pathPart, err := base64.StdEncoding.DecodeString(r.URL.Path[5:])
		if err != nil {
			this.RenderError(w, 400, err)
			return
		}

		this.RenderFile(w, string(pathPart))

	} else {
		this.RenderError(w, 404, nil)

	}

}

func (this *WebdirApp) Go() {
	http.HandleFunc("/", this.WebRequest)

	log.Printf("Serving '%s' on '%s'...", this.Basedir, this.ListenAddr)
	http.ListenAndServe(this.ListenAddr, http.DefaultServeMux)
}

func main() {

	baseDir := flag.String("Basedir", "", "Directory to serve")
	listenAddr := flag.String("Listen", "127.0.0.1:8090", "Address:port")
	cacheMax := flag.Int("CacheSize", DEFAULT_THUMB_CACHE_LIMIT, "Number of thumbnails to cache in memory")

	flag.Parse()

	if len(*baseDir) == 0 {
		cwd, err := os.Getwd()
		if err != nil {
			fmt.Fprintf(os.Stderr, "%s", err.Error())
			os.Exit(1)
		}

		*baseDir = cwd
	}

	realBase, err := filepath.Abs(*baseDir)
	if err != nil {
		fmt.Fprintf(os.Stderr, "%s", err.Error())
		os.Exit(1)
	}

	realBase = strings.TrimRight(realBase, `\`)

	wda := WebdirApp{
		Basedir:     realBase,
		ListenAddr:  *listenAddr,
		Thumbnailer: thumbnail.NewThumbnailer(100, 100, *cacheMax),
		ExcludeComponents: map[string]struct{}{
			"thumbs.db":   struct{}{},
			"desktop.ini": struct{}{},
		},
		StaticResources: map[string]struct{}{
			"app.css":    struct{}{},
			"file.png":   struct{}{},
			"folder.png": struct{}{},
		},
	}
	wda.Go()

}
